@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -49,8 +49,6 @@ FSP_HEADER
 /**********************************************************************************************************************
  * Macro definitions
  **********************************************************************************************************************/
-#define SDMMC_API_VERSION_MAJOR    (1U) // DEPRECATED
-#define SDMMC_API_VERSION_MINOR    (1U) // DEPRECATED
 
 /**********************************************************************************************************************
  * Typedef definitions
@@ -140,6 +138,8 @@ typedef enum e_sdmmc_r1_state
 typedef union u_sdmmc_response
 {
     uint32_t status;
+
+    /** SDIO Card Status Register. */
     struct
     {
         uint32_t                : 3;
@@ -171,7 +171,7 @@ typedef union u_sdmmc_response
         uint32_t block_len_error           : 1; // The transferred block length is not allowed for this card, or the number of transferred bytes does not match the block length.
         uint32_t address_error             : 1; // A misaligned address which did not match the block length was used in the command.
         uint32_t out_of_range              : 1; // The command's argument was out of the allowed range for this card.
-    };
+    } status_b;
 
     struct
     {
@@ -274,6 +274,37 @@ typedef struct st_sdmmc_callback_args
     void const     * p_context;        ///< Placeholder for user data.
 } sdmmc_callback_args_t;
 
+/** Non-secure arguments for writeIo guard function */
+typedef struct st_sdmmc_write_io_args_t
+{
+    uint8_t * const       p_data;
+    uint32_t              function;
+    uint32_t              address;
+    sdmmc_io_write_mode_t read_after_write;
+} sdmmc_write_io_args_t;
+
+/** Non-secure arguments for readIoExt guard function */
+typedef struct st_sdmmc_read_io_ext_args_t
+{
+    uint8_t * const          p_dest;
+    uint32_t                 function;
+    uint32_t                 address;
+    uint32_t * const         count;
+    sdmmc_io_transfer_mode_t transfer_mode;
+    sdmmc_io_address_mode_t  address_mode;
+} sdmmc_read_io_ext_args_t;
+
+/** Non-secure arguments for writeIoExt guard function */
+typedef struct st_sdmmc_write_io_ext_args_t
+{
+    uint8_t const * const    p_source;
+    uint32_t                 function;
+    uint32_t                 address;
+    uint32_t                 count;
+    sdmmc_io_transfer_mode_t transfer_mode;
+    sdmmc_io_address_mode_t  address_mode;
+} sdmmc_write_io_ext_args_t;
+
 /** SD/MMC Configuration */
 typedef struct st_sdmmc_cfg
 {
@@ -489,15 +520,6 @@ typedef struct st_sdmmc_api
      * @param[in]     p_ctrl    Pointer to an open SD/MMC instance control block.
      */
     fsp_err_t (* close)(sdmmc_ctrl_t * const p_ctrl);
-
-    /* DEPRECATED Returns the version of the SD/MMC driver.
-     *
-     * @par Implemented as
-     * - @ref R_SDHI_VersionGet()
-     *
-     * @param[out]  p_version       Pointer to return version information to.
-     */
-    fsp_err_t (* versionGet)(fsp_version_t * const p_version);
 } sdmmc_api_t;
 
 /** This structure encompasses everything that is needed to use an instance of this interface. */
