@@ -1,5 +1,5 @@
 metadata(version="0.2.0", description="On-device package installer for network-capable boards")
 
-require("urequests")
+require("requests")
 
 package("mip", opt=3)
