@@ -30,10 +30,6 @@
 #define MICROPY_PY_RANDOM_SEED_INIT_FUNC (trng_random_u32())
 unsigned long trng_random_u32(void);
 
-#ifndef MICROPY_PY_MACHINE_PIN_BOARD_CPU
-#define MICROPY_PY_MACHINE_PIN_BOARD_CPU (1)
-#endif
-
 // fatfs configuration used in ffconf.h
 #define MICROPY_FATFS_ENABLE_LFN            (1)
 #define MICROPY_FATFS_RPATH                 (2)
